/*
 ********************************************************************************
 *
 *  M O U S E B T N . C
 *
 *  Purpose:
 *      Demonstrate how a VxD driver can call into SYS driver in order to preserve 
 *      existing VxD API interface
 *
 *      This is the main file for the MouseBtn.VxD driver that passes calls
 *      from the legacy app (mousemon.exe) to mapsys.sys via the VXDCALL 
 *      mechanism
 *
 *  Copyright 1997  Microsoft Corp.
 *
 *  Author:     Dr. Iver
 *
 *
 ********************************************************************************
 */

#include "mousebtn.h"
#include "mousemon.h"

/**************************************************************
 *  DeviceIOControl
 *      This function gets called when the app Opens (CreateFile), 
 *      Closes (CloseHandle) or access the virtual device by the
 *      DeviceIoControl API
 *
 **************************************************************/

DWORD _stdcall DeviceIOControl(DWORD  dwService,
                               DWORD  dwDDB,
                               DWORD  hDevice,
                               PDIOCPARAMETERS lpDIOCParms )
{
    DWORD dwRetVal = 0;
    static HANDLE h;
    UINT u;
    
    //
    //  If app called CreateFile
    //
    if ( DIOC_OPEN == dwService )
    {
        //
        // Pass the call onto the MAPSYS_OpenMouse vxd interface in
        // mapsys.sys via VXDCALL.  For details, consult mousebtn.h
        //
        if  ( (h=MAPSYS_OpenMouse()) == NULL )
        {
            DBGERR(( "MAPSYS_OpenMouse failed" ));
            return( STATUS_INSUFFICIENT_RESOURCES );
        }
    }

    //
    // Else If app called CloseHandle
    // 
    else if ( DIOC_CLOSEHANDLE == dwService )
    {
        //
        // Issue close request via VXDCALL
        //
        MAPSYS_CloseMouse(h);
        dwRetVal = STATUS_SUCCESS;
    }

    //
    // Else if app called DeviceIoControl requesting button status
    //
    else if ( DIOC_GETMOUSEBUTTON == dwService )
    {
        NTSTATUS ntStatus;

        //
        // Issue read request to mapsys.sys via vxd call
        //
        ntStatus = MAPSYS_ReadButtons( h, &u );
        if ( STATUS_SUCCESS != ntStatus )
        {
            DBGERR(( "Error %x calling MAPSYS_ReadButtons",  ntStatus))
            return (DWORD)ntStatus;
        }
        //
        // store button status in buffer supplied by app
        //
        RtlCopyMemory( (PVOID)lpDIOCParms->lpvOutBuffer, 
                       (PVOID)&u,
                       sizeof(UINT) );
        lpDIOCParms->lpcbBytesReturned = sizeof( UINT );
    }
    else
    {
        DBGOUT(( "Ignorning unknown service request %x", dwService ));
        dwRetVal = STATUS_INVALID_PARAMETER;
    }
    return(dwRetVal);
}


